#!/usr/bin/env python

import os
import sys
from shutil import rmtree

from setuptools import Command, setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
VERSION = __import__('jackal').__version__


def read(f):
    return open(f, 'r', encoding='utf-8').read()


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
            rmtree(os.path.join(here, 'build'))
            rmtree(os.path.join(here, 'jackal.egg-info'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPI via Twine…')
        os.system('twine upload dist/*')
        sys.exit()


setup(
    name='django_jackal',
    version=VERSION,
    description='Boilerplate for Django and Django REST Framework',
    long_description=read('README.rst'),
    long_description_content_type='text/x-rst',
    url='https://github.com/joyongjin/jackal',
    author='Yongjin Jo',
    author_email='wnrhd114@gmail.com',
    lisence='MIT',
    packages=find_packages(exclude=['tests*', '.*']),
    zip_safe=False,
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.5",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'django>=2.0', 'djangorestframework',
    ],
    python_requires='>=3.5',
    cmdclass={
        'upload': UploadCommand,
    }
)



class BindMixin:
    bound_fields = []
    bind_field_name = 'extra'

    def __init__(self, *args, **kwargs):
        bind_data = {}
        for bound_field in self.bound_fields:
            value = kwargs.pop(bound_field, None)
            if value is not None:
                bind_data[bound_field] = value
        kwargs[self.bind_field_name] = bind_data
        super().__init__(*args, **kwargs)

    def __setattr__(self, key, value):
        if key in ['bound_fields', 'bind_field_name']:
            super().__setattr__(key, value)
        if key not in self.bound_fields or key == self.bind_field_name:
            super().__setattr__(key, value)
        else:
            bind_hash = getattr(self, self.bind_field_name, dict())
            bind_hash[key] = value
            setattr(self, self.bind_field_name, bind_hash)

    def __getattribute__(self, item):
        if item in ['bound_fields', 'bind_field_name']:
            return super().__getattribute__(item)
        elif item not in self.bound_fields or item == self.bind_field_name:
            return super().__getattribute__(item)
        else:
            bind_hash = getattr(self, self.bind_field_name, dict())
            return bind_hash.get(item)
